@@ -1,17 +1,14 @@
-/*$Id: dateline.c,v 1.2 1999/10/09 17:44:37 lindberg Exp $*/
-/*$Name: ezmlm-idx-040 $*/
-
 #include "fmt.h"
 #include "yyyymm.h"
 #include "stralloc.h"
+#include "cgi.h"
 
 static char strnum[FMT_ULONG];
 
-int dateline(dt,d)
-stralloc *dt; unsigned long d;
+int dateline(stralloc *dt, unsigned long d)
 /* converts yyyymm from unsigned long d to text dt */
 {
-  char *mo;
+  const char *mo;
   switch (d % 100) {
     case 1: mo = "January"; break;
     case 2: mo = "February"; break;
