@@ -1,11 +1,14 @@
-/* $Id: ezmlm-send.c,v 1.77 1999/10/29 02:49:14 lindberg Exp $*/
-/* $Name: ezmlm-idx-040 $*/
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <stdio.h>
+#include <unistd.h>
 #include "stralloc.h"
 #include "subfd.h"
 #include "strerr.h"
 #include "error.h"
 #include "qmail.h"
 #include "env.h"
+#include "sender.h"
 #include "lock.h"
 #include "sig.h"
 #include "open.h"
@@ -20,31 +23,48 @@
 #include "getconf.h"
 #include "constmap.h"
 #include "byte.h"
-#include "sgetopt.h"
+#include "getconfopt.h"
 #include "quote.h"
-#include "subscribe.h"
+#include "subdb.h"
 #include "mime.h"
-#include "errtxt.h"
+#include "messages.h"
 #include "makehash.h"
 #include "cookie.h"
+#include "hdr.h"
+#include "die.h"
+#include "wrap.h"
 #include "idx.h"
 #include "copy.h"
+#include "config.h"
+#include "auto_version.h"
 
 int flagnoreceived = 1;		/* suppress received headers by default. They*/
 				/* are still archived. =0 => archived and */
 				/* copied. */
 int flaglog = 1;		/* for lists with mysql support, use tags */
 				/* and log traffic to the database */
-#define FATAL "ezmlm-send: fatal: "
+const char FATAL[] = "ezmlm-send: fatal: ";
+const char USAGE[] =
+"ezmlm-send: usage: ezmlm-send [-cClLqQrR] [-h header] dir";
 
-void die_usage()
-{
-  strerr_die1x(100,"ezmlm-send: usage: ezmlm-send [-cClLqQrR] [-h header] dir");
-}
-void die_nomem()
-{
-  strerr_die2x(111,FATAL,ERR_NOMEM);
-}
+static int ignoredflag;
+static const char *hash_str = 0;
+static char *mlheader = 0;
+
+static struct option options[] = {
+  OPT_FLAG(ignoredflag,'c',0,0), /* ignore for backwards compat */
+  OPT_FLAG(ignoredflag,'C',0,0),
+  OPT_CSTR(mlheader,'H',0),	/* Alternative sublist check header */
+  OPT_FLAG(flaglog,'l',1,0),
+  OPT_FLAG(flaglog,'L',0,0),
+  OPT_FLAG(flagnoreceived,'r',0,0),
+  OPT_FLAG(flagnoreceived,'R',1,0),
+  OPT_CSTR(hash_str,'s',0),
+  OPT_CSTR(hash_str,'S',0),
+  OPT_FLAG(ignoredflag,'q',0,0),
+  OPT_FLAG(ignoredflag,'Q',0,0),
+  OPT_END
+};
 
 	/* for writing new index file indexn later moved to index. */
 substdio ssindexn;
@@ -68,32 +88,27 @@ stralloc received = {0};
 stralloc prefix = {0};
 stralloc content = {0};
 stralloc boundary = {0};
-stralloc charset = {0};
 stralloc dcprefix = {0};
 stralloc dummy = {0};
-stralloc qmqpservers = {0};
 
-void die_indexn()
+void die_indexn(void)
 {
-  strerr_die4x(111,FATAL,ERR_WRITE,fnifn.s,": ");
+  strerr_die2sys(111,FATAL,MSG1(ERR_WRITE,fnifn.s));
 }
 
-void *psql = (void *) 0;
-
 unsigned long innum;
 unsigned long outnum;
-unsigned long msgnum;
+unsigned long msgnum = 0L;
 unsigned long hash_lo = 0L;
 unsigned long hash_hi = 52L;
 unsigned long msgsize = 0L;
 unsigned long cumsize = 0L;	/* cumulative archive size bytes / 256 */
-char flagcd = '\0';		/* no transfer-encoding for trailer */
 char encin = '\0';
 int flagindexed;
 int flagfoundokpart;		/* Found something to pass on. If multipart */
 				/* we set to 0 and then set to 1 for any */
 				/* acceptable mime part. If 0 -> reject */
-int flagreceived;
+int flagsawreceived;
 int flagprefixed;
 unsigned int serial = 0;
 int flagarchived;
@@ -107,36 +122,21 @@ char archivebuf[1024];
 
 int flagsublist;
 stralloc sublist = {0};
-stralloc mailinglist = {0};
-stralloc outlocal = {0};
-stralloc outhost = {0};
 stralloc headerremove = {0};
 struct constmap headerremovemap;
+int headerremoveflag = 0;
 stralloc mimeremove = {0};
 struct constmap mimeremovemap;
-char *dir;
+int mimeremoveflag = 0;
 
 struct qmail qq;
 substdio ssin;
 char inbuf[1024];
-substdio ssout;
-char outbuf[1];
 
 char textbuf[512];
 substdio sstext;
 
-unsigned int mywrite(fd,buf,len)
-int fd;
-char *buf;
-unsigned int len;
-{
-  qmail_put(&qq,buf,len);
-  return len;
-}
-
-int subto(s,l)
-char *s;
-unsigned int l;
+int subto(const char *s,unsigned int l)
 {
   qmail_put(&qq,"T",1);
   qmail_put(&qq,s,l);
@@ -144,31 +144,30 @@ unsigned int l;
   return (int) l;
 }
 
-void die_archive()
+void die_archive(void)
 {
-  strerr_die4sys(111,FATAL,ERR_WRITE,fnaf.s,": ");
+  strerr_die2sys(111,FATAL,MSG1(ERR_WRITE,fnaf.s));
 }
-void die_numnew()
+void die_numnew(void)
 {
-  strerr_die3sys(111,FATAL,ERR_CREATE,"numnew: ");
+  strerr_die2sys(111,FATAL,MSG1(ERR_CREATE,"numnew"));
 }
 
-void qa_put(buf,len) char *buf; unsigned int len;
+void qa_put(const char *buf,unsigned int len)
 {
   qmail_put(&qq,buf,len);
   if (flagarchived)
     if (substdio_put(&ssarchive,buf,len) == -1) die_archive();
 }
 
-void qa_puts(buf) char *buf;
+void qa_puts(const char *buf)
 {
   qmail_puts(&qq,buf);
   if (flagarchived)
     if (substdio_puts(&ssarchive,buf) == -1) die_archive();
 }
 
-int sublistmatch(sender)
-char *sender;
+int sublistmatch(const char *sender)
 {
   unsigned int i;
   unsigned int j;
@@ -189,10 +188,7 @@ char *sender;
 substdio ssnumnew;
 char numnewbuf[16];
 
-char buf0[256];
-substdio ss0 = SUBSTDIO_FDBUF(read,0,buf0,sizeof(buf0));
-
-void numwrite()
+void numwrite(void)
 {		/* this one deals with msgnum, not outnum! */
   int fd;
 
@@ -209,13 +205,12 @@ void numwrite()
   if (substdio_flush(&ssnumnew) == -1) die_numnew();
   if (fsync(fd) == -1) die_numnew();
   if (close(fd) == -1) die_numnew(); /* NFS stupidity */
-  if (rename("numnew","num") == -1)
-    strerr_die3sys(111,FATAL,ERR_MOVE,"numnew: ");
+  wrap_rename("numnew","num");
 }
 
 stralloc mydtline = {0};
 
-int idx_copy_insertsubject()
+int idx_copy_insertsubject(void)
 /* copies old index file up to but not including msg, then adds a line with */
 /* 'sub' trimmed of reply indicators, then closes the new index and moves it*/
 /* to the name 'index'. Errors are dealt with directly, and if the routine  */
@@ -229,6 +224,7 @@ int idx_copy_insertsubject()
   int match;
   int r;
   unsigned int pos;
+  int k;
 
   if (!stralloc_copys(&fnadir,"archive/")) die_nomem();
   if (!stralloc_catb(&fnadir,strnum,fmt_ulong(strnum,outnum / 100)))
@@ -244,44 +240,44 @@ int idx_copy_insertsubject()
 			/* may not exists since we run before ezmlm-send */
   if (mkdir(fnadir.s,0755) == -1)
     if (errno != error_exist)
-      strerr_die4x(111,FATAL,ERR_CREATE,fnadir.s,": ");
+      strerr_die2sys(111,FATAL,MSG1(ERR_CREATE,fnadir.s));
 
 			/* Open indexn */
   fdindexn = open_trunc(fnifn.s);
   if (fdindexn == -1)
-    strerr_die4x(111,FATAL,ERR_WRITE,fnifn.s,": ");
+    strerr_die2sys(111,FATAL,MSG1(ERR_WRITE,fnifn.s));
 
 			/* set up buffers for indexn */
   substdio_fdbuf(&ssindexn,write,fdindexn,indexnbuf,sizeof(indexnbuf));
 
-  concatHDR(subject.s,subject.len,&lines,FATAL);	/* make 1 line */
-  decodeHDR(lines.s,lines.len,&qline,charset.s,FATAL);	/* decode mime */
-  r = unfoldHDR(qline.s,qline.len,&lines,charset.s,&dcprefix,1,FATAL);
+  concatHDR(subject.s,subject.len,&lines);	/* make 1 line */
+  decodeHDR(lines.s,lines.len,&qline);		/* decode mime */
+  r = unfoldHDR(qline.s,qline.len,&lines,charset.s,&dcprefix,1);
 						 /* trim mime */
 
   fdindex = open_read(fnif.s);
   if (fdindex == -1) {
     if (errno != error_noent)
-      strerr_die4x(111,FATAL,ERR_OPEN, fnif.s, ": ");
+      strerr_die2x(111,FATAL,MSG1(ERR_OPEN, fnif.s));
   } else {
     substdio_fdbuf(&ssin,read,fdindex,inbuf,sizeof(inbuf));
     for(;;) {
       if (getln(&ssin,&qline,&match,'\n') == -1)
-        strerr_die4sys(111,FATAL,ERR_READ, fnif.s, ": ");
+        strerr_die2sys(111,FATAL,MSG1(ERR_READ,fnif.s));
       if (!match)
         break;
       pos = scan_ulong(qline.s,&idx);
       if (!idx)				/* "impossible!" */
-        strerr_die2x(111,FATAL,ERR_BAD_INDEX);
+        strerr_die2x(111,FATAL,MSG(ERR_BAD_INDEX));
       if (idx >= outnum)
         break;				/* messages always come in order */
       if (substdio_put(&ssindexn,qline.s,qline.len) == -1)
         die_indexn();
       if (qline.s[pos] == ':') {	/* has author line */
         if (getln(&ssin,&qline,&match,'\n') == -1)
-          strerr_die4x(111,FATAL,ERR_READ, fnif.s, ": ");
+          strerr_die2sys(111,FATAL,MSG1(ERR_READ,fnif.s));
         if (!match && qline.s[0] != '\t')	/* "impossible! */
-          strerr_die2x(111,FATAL,ERR_BAD_INDEX);
+          strerr_die2x(111,FATAL,MSG(ERR_BAD_INDEX));
         if (substdio_put(&ssindexn,qline.s,qline.len) == -1)
           die_indexn();
       }
@@ -300,14 +296,16 @@ int idx_copy_insertsubject()
   if (!stralloc_cat(&qline,&received)) die_nomem();
   if (!stralloc_cats(&qline,";")) die_nomem();
 
-  concatHDR(from.s,from.len,&lines,FATAL);
+  concatHDR(from.s,from.len,&lines);
   mkauthhash(lines.s,lines.len,hash);
 
   if (!stralloc_catb(&qline,hash,HASHLEN)) die_nomem();
   if (!stralloc_cats(&qline," ")) die_nomem();
 
-  decodeHDR(cp,author_name(&cp,lines.s,lines.len),&from,charset.s,FATAL);
-  (void) unfoldHDR(from.s,from.len,&lines,charset.s,&dcprefix,0,FATAL);
+  k = author_name(&cp,lines.s,lines.len);
+  decodeHDR(cp,k,&from);
+
+  (void) unfoldHDR(from.s,from.len,&lines,charset.s,&dcprefix,0);
   if (!stralloc_cat(&qline,&lines)) die_nomem();
 
   if (!stralloc_cats(&qline,"\n")) die_nomem();
@@ -316,160 +314,83 @@ int idx_copy_insertsubject()
   if (fsync(fdindexn) == -1) die_indexn();
   if (fchmod(fdindexn,MODE_ARCHIVE | 0700) == -1) die_indexn();
   if (close(fdindexn) == -1) die_indexn(); /* NFS stupidity */
-  if (rename(fnifn.s,fnif.s) == -1)
-    strerr_die4x(111,FATAL,ERR_MOVE,fnifn.s,": ");
+  wrap_rename(fnifn.s,fnif.s);
   return r;
 }
 
-void transferenc()
-{
-	if (flagcd) {
-	  qmail_puts(&qq,"\nContent-Transfer-Encoding: ");
-          if (flagcd == 'Q')
-            qmail_puts(&qq,"Quoted-printable\n\n");
-          else
-	    qmail_puts(&qq,"base64\n\n");
-        } else
-          qmail_puts(&qq,"\n\n");
-}
-
-void getcharset()
-{
-    if (getconf_line(&charset,"charset",0,FATAL,dir)) {
-      if (charset.len >= 2 && charset.s[charset.len - 2] == ':') {
-        if (charset.s[charset.len - 1] == 'B' ||
-			charset.s[charset.len - 1] == 'Q') {
-          flagcd = charset.s[charset.len - 1];
-          charset.s[charset.len - 2] = '\0';
-        }
-      }
-    } else
-      if (!stralloc_copys(&charset,TXT_DEF_CHARSET)) die_nomem();
-
-    if (!stralloc_0(&charset)) die_nomem();
-}
-
-void main(argc,argv)
-int argc;
-char **argv;
+int main(int argc,char **argv)
 {
   unsigned long subs;
   int fdlock;
-  char *sender;
-  char *mlheader = (char *) 0;
-  char *ret;
-  char *err;
+  const char *sender;
+  const char *ret;
+  const char *err;
   int flagmlwasthere;
-  int flagqmqp = 0;	/* don't use qmqp by default */
   int flaglistid = 0;	/* no listid header added */
   int match;
   unsigned int i;
-  int r,fd;
+  int r = 0;
   int flaginheader;
   int flagbadfield;
   int flagbadpart;
   int flagseenext;
-  int flagsubline;
-  int flagfromline;
-  int flagcontline;
   int flagarchiveonly;
   int flagtrailer;
+  enum { no_prev_line, subject_line, from_line, content_line, received_line } prevline;
   unsigned int pos;
-  int opt;
   char *cp, *cpstart, *cpafter;
 
   umask(022);
   sig_pipeignore();
 
-  while ((opt = getopt(argc,argv,"cCh:H:lLrRqQs:S:vV")) != opteof)
-    switch(opt) {
-      case 'c': case 'C': break;	/* ignore for backwards compat */
-      case 'h':
-      case 'H': mlheader = optarg;	/* Alternative sublist check header */
-                mlheader[str_chr(mlheader,':')] = '\0';
-                break;
-      case 'l': flaglog = 1; break;
-      case 'L': flaglog = 0; break;
-      case 'r': flagnoreceived = 0; break;
-      case 'R': flagnoreceived = 1; break;
-      case 's':
-      case 'S':	pos = scan_ulong(optarg,&hash_lo);
-		if (!optarg[pos++]) break;
-		(void) scan_ulong(optarg+pos,&hash_hi);
-		if (hash_hi > 52L) hash_hi = 52L;
-		if (hash_lo > hash_hi) hash_lo = hash_hi;
-
- break;
-      case 'q': flagqmqp = 0; break;
-      case 'Q': flagqmqp = 1; break;
-      case 'v':
-      case 'V': strerr_die2x(0,
-		"ezmlm-send version: ezmlm-0.53+",EZIDX_VERSION);
-      default:
-	die_usage();
+  getconfopt(argc,argv,options,1,0);
+  /* Fix up some command-line arguments. */
+  if (mlheader != 0)
+    mlheader[str_chr(mlheader,':')] = '\0';
+  if (hash_str != 0) {
+    pos = scan_ulong(hash_str,&hash_lo);
+    if (hash_str[pos++]) {
+      (void) scan_ulong(hash_str+pos,&hash_hi);
+      if (hash_hi > 52L)
+	hash_hi = 52L;
+      if (hash_lo > hash_hi)
+	hash_lo = hash_hi;
     }
+  }
+  initsub(0);
 
+  sender = get_sender();
 
-  dir = argv[optind++];
-  if (!dir) die_usage();
+  fdlock = lockfile("lock");
 
-  sender = env_get("SENDER");
-
-  if (chdir(dir) == -1)
-    strerr_die4sys(111,FATAL,ERR_SWITCH,dir,": ");
-
-  fdlock = open_append("lock");
-  if (fdlock == -1)
-    strerr_die4sys(111,FATAL,ERR_OPEN,dir,"/lock: ");
-  if (lock_ex(fdlock) == -1)
-    strerr_die4sys(111,FATAL,ERR_OBTAIN,dir,"/lock: ");
-
-  flagarchived = getconf_line(&line,"archived",0,FATAL,dir);
-  flagindexed = getconf_line(&line,"indexed",0,FATAL,dir);
-  getcharset();
-  flagprefixed = getconf_line(&prefix,"prefix",0,FATAL,dir);
+  flagarchived = getconf_isset("archived");
+  flagindexed = getconf_isset("indexed");
+  flagprefixed = getconf_line(&prefix,"prefix",0);
   if (prefix.len) {		/* encoding and serial # support */
 				/* no sanity checks - you put '\n' or '\0' */
 				/* into the coded string, you pay */
 
-    decodeHDR(prefix.s,prefix.len,&line,charset.s,FATAL);
-    (void) unfoldHDR(line.s,line.len,&dcprefix,charset.s,&dummy,0,FATAL);
+    decodeHDR(prefix.s,prefix.len,&line);
+    (void) unfoldHDR(line.s,line.len,&dcprefix,charset.s,&dummy,0);
     if (!stralloc_copy(&dcprefix,&line)) die_nomem();
     serial = byte_rchr(prefix.s,prefix.len,'#');
   }
-  if ((fd = open_read("text/trailer")) == -1) {	/* see if there is a trailer */
-    if (errno == error_noent) flagtrailer = 0;
-    else strerr_die2sys(111,ERR_OPEN,"text/trailer: ");
-  } else {
-    close(fd);
-    flagtrailer = 1;
-  }
+  flagtrailer = getconf_isset("addtrailer");
 
-  getconf(&mimeremove,"mimeremove",0,FATAL,dir);
+  if (getconf(&mimeremove,"mimekeep",0))
+    mimeremoveflag = 1;
+  else
+    getconf(&mimeremove,"mimeremove",0);
 
-  if (getconf_line(&line,"num",0,FATAL,dir)) {	/* Now non-FATAL, def=0 */
-    if (!stralloc_0(&line)) die_nomem();
-    cp = line.s + scan_ulong(line.s,&msgnum);
-    ++msgnum;
-    if (*cp++ == ':')
-      scan_ulong(cp,&cumsize);
-  } else
-    msgnum = 1L;			/* if num not there */
+  getconf_ulong2(&msgnum,&cumsize,"num",0);
+  ++msgnum;
 
-  getconf_line(&outhost,"outhost",1,FATAL,dir);
-  getconf_line(&outlocal,"outlocal",1,FATAL,dir);
-  set_cpoutlocal(&outlocal);
-  set_cpouthost(&outhost);
-  flagsublist = getconf_line(&sublist,"sublist",0,FATAL,dir);
+  flagsublist = getconf_line(&sublist,"sublist",0);
 
-  if (flagqmqp) {			/* forward compatibility ;-) */
-    if (!stralloc_copys(&line,QMQPSERVERS)) die_nomem();
-    if (!stralloc_cats(&line,"/0")) die_nomem();
-    if (!stralloc_0(&line)) die_nomem();
-    (void) getconf_line(&qmqpservers,line.s,0,FATAL,dir);
-  }
-
-  getconf(&headerremove,"headerremove",1,FATAL,dir);
+  if (getconf(&headerremove,"headerkeep",0))
+    headerremoveflag = 1;
+  else
+    getconf(&headerremove,"headerremove",1);
   if (!constmap_init(&headerremovemap,headerremove.s,headerremove.len,0))
 	die_nomem();
 
@@ -481,12 +402,12 @@ char **argv;
 
   if (sender) {
     if (!*sender)
-      strerr_die2x(100,FATAL,ERR_BOUNCE);
+      strerr_die2x(100,FATAL,MSG(ERR_BOUNCE));
     if (str_equal(sender,"#@[]"))
-      strerr_die2x(100,FATAL,ERR_BOUNCE);
+      strerr_die2x(100,FATAL,MSG(ERR_BOUNCE));
     if (flagsublist)
       if (!sublistmatch(sender))
-        strerr_die2x(100,FATAL,ERR_NOT_PARENT);
+        strerr_die2x(100,FATAL,MSG(ERR_NOT_PARENT));
   }
   innum = msgnum;				/* innum = incoming */
   outnum = msgnum;				/* outnum = outgoing */
@@ -514,10 +435,10 @@ char **argv;
 
     if (mkdir(fnadir.s,0755) == -1)
       if (errno != error_exist)
-	strerr_die4sys(111,FATAL,ERR_CREATE,fnadir.s,": ");
+	strerr_die2sys(111,FATAL,MSG1(ERR_CREATE,fnadir.s));
     fdarchive = open_trunc(fnaf.s);
     if (fdarchive == -1)
-      strerr_die4sys(111,FATAL,ERR_WRITE,fnaf.s,": ");
+      strerr_die2sys(111,FATAL,MSG1(ERR_WRITE,fnaf.s));
 
     substdio_fdbuf(&ssarchive,write,fdarchive,archivebuf,sizeof(archivebuf));
 						/* return-path to archive */
@@ -531,38 +452,31 @@ char **argv;
     if (substdio_put(&ssarchive,line.s,line.len) == -1) die_archive();
   }
 
-  if (flagqmqp) {
-    if (qmail_open(&qq,&qmqpservers) == -1)		/* open qmqp */
-      strerr_die2sys(111,FATAL,ERR_QMAIL_QUEUE);
-  } else if (qmail_open(&qq,(stralloc *) 0) == -1)	/* open queue */
-      strerr_die2sys(111,FATAL,ERR_QMAIL_QUEUE);
+    if (qmail_open(&qq) == -1)
+      strerr_die2sys(111,FATAL,MSG(ERR_QMAIL_QUEUE));
 
   if (!flagsublist) {
-    getconf_line(&mailinglist,"mailinglist",1,FATAL,dir);
-    qa_puts("Mailing-List: ");
-    qa_put(mailinglist.s,mailinglist.len);
-    if (getconf_line(&line,"listid",0,FATAL,dir)) {
+    hdr_add2s("Mailing-List: ",MSG(TXT_MAILING_LIST));
+    if (listid.len > 0) {
       flaglistid = 1;
-      qmail_puts(&qq,"\nList-ID: ");
-      qmail_put(&qq,line.s,line.len);
+      hdr_add2("List-ID: ",listid.s,listid.len);
     }
-    qa_puts("\n");
   }
-  copy(&qq,"headeradd",'H',FATAL);
+  copy(&qq,"headeradd",'H');
   qa_put(mydtline.s,mydtline.len);
 
   flagmlwasthere = 0;
   flaginheader = 1;
   flagfoundokpart = 1;
-  flagbadfield = 0;
+  flagbadfield = headerremoveflag;
   flagbadpart = 0;
   flagseenext = 0;
-  flagsubline = 0;
-  flagfromline = 0;
-  flagreceived = 0;
+  flagsawreceived = 0;
+  flagarchiveonly = 0;
+  prevline = no_prev_line;
   for (;;) {
-    if (getln(&ss0,&line,&match,'\n') == -1)
-      strerr_die2sys(111,FATAL,ERR_READ_INPUT);
+    if (getln(subfdin,&line,&match,'\n') == -1)
+      strerr_die2sys(111,FATAL,MSG(ERR_READ_INPUT));
     if (flaginheader && match) {
       if (line.len == 1) {		/* end of header */
 	flaginheader = 0;
@@ -571,10 +485,10 @@ char **argv;
         if (flagprefixed && !flagsublist) {
           qa_puts("Subject:");
           if (!flagindexed) {		/* non-indexed prefixed lists */
-            concatHDR(subject.s,subject.len,&lines,FATAL);
-            decodeHDR(lines.s,lines.len,&qline,charset.s,FATAL);
+            concatHDR(subject.s,subject.len,&lines);
+            decodeHDR(lines.s,lines.len,&qline);
             r = unfoldHDR(qline.s,qline.len,&lines,
-			charset.s,&dcprefix,1,FATAL);
+			charset.s,&dcprefix,1);
           }
           if (!(r & 2)) {
             qmail_puts(&qq," ");
@@ -590,7 +504,7 @@ char **argv;
         }
 		/* do other stuff to do with post header processing here */
 	if (content.len) {		/* get MIME boundary, if exists */
-          concatHDR(content.s,content.len,&qline,FATAL);
+          concatHDR(content.s,content.len,&qline);
           if (!stralloc_copy(&content,&qline)) die_nomem();
 
 	  if (flagtrailer &&		/* trailer only for some multipart */
@@ -613,7 +527,7 @@ char **argv;
                  cpstart = cp;
                  while (cp < cpafter && *cp != '"') ++cp;
 		 if (cp == cpafter)
-			strerr_die1x(100,ERR_MIME_QUOTE);
+			strerr_die1x(100,MSG(ERR_MIME_QUOTE));
                } else {			/* non-quoted boundary */
                  cpstart = cp;		/* find terminator of boundary */
                  while (cp < cpafter && *cp != ';' &&
@@ -622,7 +536,7 @@ char **argv;
                if (!stralloc_copys(&boundary,"--")) die_nomem();
                if (!stralloc_catb(&boundary,cpstart,cp-cpstart))
 			die_nomem();
-	         flagfoundokpart = 0;
+	       flagfoundokpart = 0;
                if (!constmap_init(&mimeremovemap,mimeremove.s,mimeremove.len,0))
 			die_nomem();
                flagbadpart = 1;		/* skip before first boundary */
@@ -631,21 +545,24 @@ char **argv;
           }
         }
       } else if ((*line.s != ' ') && (*line.s != '\t')) {
-        flagsubline = 0;
-        flagfromline = 0;
-        flagbadfield = 0;
+        flagbadfield = headerremoveflag;
         flagarchiveonly = 0;
-        flagcontline = 0;
+	prevline = no_prev_line;
 	if (constmap(&headerremovemap,line.s,byte_chr(line.s,line.len,':')))
-	  flagbadfield = 1;
-        if ((flagnoreceived || !flagreceived) &&
+	  flagbadfield = !headerremoveflag;
+        if ((flagnoreceived || !flagsawreceived) &&
 		case_startb(line.s,line.len,"Received:")) {
-            if (!flagreceived) {		/* get date from first rec'd */
-              flagreceived = 1;			/* line (done by qmail) */
+            if (!flagsawreceived) {		/* get date from first rec'd */
+              flagsawreceived = 1;		/* line (done by qmail) */
               pos = byte_chr(line.s,line.len,';');
-              if (pos != line.len)		/* has '\n' */
-                if (!stralloc_copyb(&received,line.s+pos+2,line.len - pos - 3))
+              if (pos != line.len) {		/* has '\n' */
+		for (++pos; pos < line.len; ++pos)
+		  if (line.s[pos] != ' ' && line.s[pos] != '\t')
+		    break;
+                if (!stralloc_copyb(&received,line.s+pos,line.len-pos-1))
                   die_nomem();
+		prevline = received_line;
+	      }
             } else {				/* suppress, but archive */
               flagarchiveonly = 1;		/* but do not suppress the */
               flagbadfield = 1;			/* top one added by qmail */
@@ -657,10 +574,10 @@ char **argv;
         else if ((mimeremove.len || flagtrailer) &&	/* else no MIME need*/
 		case_startb(line.s,line.len,"Content-Type:")) {
           if (!stralloc_copyb(&content,line.s+13,line.len-13)) die_nomem();
-          flagcontline = 1;
+          prevline = content_line;
 	} else if (case_startb(line.s,line.len,"Subject:")) {
           if (!stralloc_copyb(&subject,line.s+8,line.len-8)) die_nomem();
-	  flagsubline = 1;
+	  prevline = subject_line;
           if (flagprefixed && !flagsublist)	/* don't prefix for sublists */
 	    flagbadfield = 1;			/* we'll print our own */
         } else if (flagtrailer &&
@@ -675,20 +592,30 @@ char **argv;
 	else if (flagindexed) {
 
           if (case_startb(line.s,line.len,"From:")) {
-            flagfromline = 1;
+            prevline = from_line;
             if (!stralloc_copyb(&from,line.s+5,line.len-5)) die_nomem();
           }
         } else if (line.len == mydtline.len)
 	  if (!byte_diff(line.s,line.len,mydtline.s))
-            strerr_die2x(100,FATAL,ERR_LOOPING);
+            strerr_die2x(100,FATAL,MSG(ERR_LOOPING));
       } else {			/* continuation lines */
-        if (flagsubline) {
+	switch (prevline) {
+	case subject_line:
 	  if (!stralloc_cat(&subject,&line)) die_nomem();
-        } else if (flagfromline) {
+	  break;
+	case from_line:
 	  if (!stralloc_cat(&from,&line)) die_nomem();
-        } else if (flagcontline) {
+	  break;
+	case content_line:
           if (!stralloc_cat(&content,&line)) die_nomem();
-        }
+	  break;
+	case received_line:
+	  for (pos = 1; pos < line.len; ++pos)
+	    if (line.s[pos] != ' ' && line.s[pos] != '\t')
+	      break;
+	  if (!stralloc_catb(&received,line.s+pos,line.len-pos-1)) die_nomem();
+	  break;
+	}
       }
     } else				/* body */
       msgsize += line.len;		/* always for tstdig support */
@@ -700,13 +627,12 @@ char **argv;
           flagbadpart = 0;		/* end boundary should be output */
           if (flagtrailer) {
             qmail_puts(&qq,"\n");
-            qmail_put(&qq,boundary.s,boundary.len);
-            qmail_puts(&qq,"\nContent-Type: text/plain; charset=");
-            qmail_puts(&qq,charset.s);
-            transferenc();		/* trailer for multipart message */
-	    copy(&qq,"text/trailer",flagcd,FATAL);
+	    hdr_add(boundary.s,boundary.len);
+	    hdr_ctype(CTYPE_TEXT);
+            hdr_transferenc();		/* trailer for multipart message */
+	    copy(&qq,"text/trailer",flagcd);
             if (flagcd == 'B')	{	/* need to do our own flushing */
-              encodeB("",0,&qline,2,FATAL);
+              encodeB("",0,&qline,2);
               qmail_put(&qq,qline.s,qline.len);
             }
 	  }
@@ -722,16 +648,19 @@ char **argv;
           while (*cp == ' ' || *cp == '\t') ++cp;
           cpstart = cp;			/* end of type */
           while (*cp != '\n' && *cp != '\t' && *cp != ' ' && *cp != ';') ++cp;
-	  if (constmap(&mimeremovemap,cpstart,cp-cpstart)) {
-            flagbadpart = 1;
-          } else {
+	  flagbadpart = constmap(&mimeremovemap,cpstart,cp-cpstart)
+	    ? !mimeremoveflag
+	    : mimeremoveflag;
+	  if (!flagbadpart) {
+	    /* do this part */
 	    flagfoundokpart = 1;
-            qa_put(lines.s,lines.len);	/* saved lines */
-            flagbadpart = 0;		/* do this part */
-          }
+	    qa_put(lines.s,lines.len); /* saved lines */
+	  }
         } else if (line.len == 1) {	/* end of content desc */
           flagbadpart = 0;		/* default type, so ok */
+          flagfoundokpart = 1;		/* this is part of a multipart msg */
           flagseenext = 0;		/* done thinking about it */
+          qa_put(lines.s,lines.len);	/* saved lines */
         } else				/* save line in cont desc */
           if (!stralloc_cat(&lines,&line)) die_nomem();
       }
@@ -744,28 +673,28 @@ char **argv;
       break;
   }
   if (!boundary.len && flagtrailer) {
-    qmail_puts(&qq,"\n");		/* trailer for non-multipart message */
-    if (!encin || encin == 'Q') {	/* can add for QP, but not for base64 */
-      copy(&qq,"text/trailer",encin,FATAL);
-      qmail_puts(&qq,"\n");		/* no need to flush for plain/QP */
-    }
+    codeput(&qq,"\n",1,encin);		/* trailer for non-multipart message */
+    copy(&qq,"text/trailer",encin);
+    codeput(&qq,"\n",1,encin);		/* no need to flush for plain/QP */
+    if (encin == 'B')
+      qmail_put(&qq,"\n",1); /* Need an actual newline after the base64 data */
   }
 
   cumsize += (msgsize + 128L) >> 8;	/* round to 256 byte 'records' */
 					/* check message tag */
   if (flagsublist) {			/* sublists need tag if selected/suppt*/
     if (flaglog)
-      if ((ret = checktag(dir,innum,hash_lo+1L,"m",(char *) 0,hashout))) {
+      if ((ret = checktag(innum,hash_lo+1L,"m",(char *) 0,hashout))) {
         if (*ret) strerr_die2x(111,FATAL,ret);
-        else strerr_die2x(100,FATAL,ERR_NOT_PARENT);
+        else strerr_die2x(100,FATAL,MSG(ERR_NOT_PARENT));
       }
     if (!flagmlwasthere)		/* sublists need ML header */
-      strerr_die2x(100,FATAL,ERR_SUBLIST);
+      strerr_die2x(100,FATAL,MSG(ERR_SUBLIST));
   } else				/* others are not allowed to have one */
     if (flagmlwasthere)
-      strerr_die2x(100,FATAL,ERR_MAILING_LIST);
+      strerr_die2x(100,FATAL,MSG(ERR_MAILING_LIST));
   if (!flagfoundokpart)			/* all parts were on the strip list */
-      strerr_die2x(100,FATAL,ERR_BAD_ALL);
+      strerr_die2x(100,FATAL,MSG(ERR_BAD_ALL));
 
   if (flagarchived) {
     if (substdio_flush(&ssarchive) == -1) die_archive();
@@ -775,7 +704,7 @@ char **argv;
   }
 
   if (flaglog) {
-    tagmsg(dir,innum,sender,"m",hashout,qq.msgbytes,53L,FATAL);
+    tagmsg(innum,sender,"m",hashout,qq.msgbytes,53L);
     hashout[COOKIE] = '\0';
   }
 
@@ -788,19 +717,19 @@ char **argv;
   if (!stralloc_cats(&line,"-@[]")) die_nomem();
   if (!stralloc_0(&line)) die_nomem();
   qmail_from(&qq,line.s);			/* envelope sender */
-  subs = putsubs(dir,hash_lo,hash_hi,subto,1,FATAL);	/* subscribers */
+  subs = putsubs(0,hash_lo,hash_hi,subto);	/* subscribers */
   if (flagsublist) hash_lo++;
 
   if (*(err = qmail_close(&qq)) == '\0') {
       if (flaglog)				/* mysql logging */
-	(void) logmsg(dir,outnum,hash_lo,subs,flagsublist ? 3 : 4);
-      closesql();
+	(void) logmsg(outnum,hash_lo,subs,flagsublist ? 3 : 4);
+      closesub();
       strnum[fmt_ulong(strnum,qmail_qp(&qq))] = 0;
       strerr_die2x(0,"ezmlm-send: info: qp ",strnum);
   } else {
       --msgnum;
       cumsize -= (msgsize + 128L) >> 8;
       numwrite();
-      strerr_die3x(111,FATAL,ERR_TMP_QMAIL_QUEUE,err + 1);
+      strerr_die4x(111,FATAL,MSG(ERR_TMP_QMAIL_QUEUE),": ",err + 1);
   }
 }
