@@ -1,6 +1,3 @@
-/*$Id: idxthread.c,v 1.35 1999/11/22 01:47:45 lindberg Exp $*/
-/*$Name: ezmlm-idx-040 $*/
-
 /* idxthread.c contains routines to from the ezmlm-idx subject index build */
 /* a structure of unique subjects as well as a table of messages with      */
 /* pointers to the subject. This leads to information on message threads   */
@@ -12,14 +9,20 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <unistd.h>
 #include "error.h"
 #include "alloc.h"
 #include "str.h"
 #include "stralloc.h"
 #include "strerr.h"
 #include "lock.h"
+#include "open.h"
+#include "getln.h"
+#include "scan.h"
+#include "byte.h"
+#include "die.h"
 #include "idx.h"
-#include "errtxt.h"
+#include "messages.h"
 #include "substdio.h"
 #include "fmt.h"
 #include "readwrite.h"
@@ -42,45 +45,28 @@ struct stat st;
 static subentry sdummy;
 static authentry adummy;
 
-
-static void die_nomem(fatal)
-char *fatal;
-{
-  strerr_die2x(111,fatal,ERR_NOMEM);
-}
-
 int fdlock;
 
 /* NOTE: These do NOT prevent double locking */
-static void lockup(fatal)
-char *fatal;
+static void lockup(void)
 {
-  fdlock = open_append("lock");
-  if (fdlock == -1)
-    strerr_die2sys(111,fatal,ERR_OPEN_LOCK);
-  if (lock_ex(fdlock) == -1) {
-    close(fdlock);
-    strerr_die2sys(111,fatal,ERR_OBTAIN_LOCK);
-  }
+    fdlock = lockfile("lock");
 }
 
-static void unlock()
+static void unlock(void)
 {
     close(fdlock);
 }
 
-static void newsub(psubt,subject,sublen,msg,fatal)
+static void newsub(subentry *psubt,const char *subject,unsigned int sublen,
+		   unsigned long msg)
 /* Initializes subentry pointed to by psubt, adds a '\0' to subject,    */
 /* allocates space and copies in subject, and puts a pointer to it in   */
 /* the entry. */
-subentry *psubt;
-char *subject;
-unsigned int sublen;
-unsigned long msg;
-char *fatal;
 {
-  register char *cpfrom, *cpto;
-  register unsigned int cpno;
+  const char *cpfrom;
+  char *cpto;
+  unsigned int cpno;
 
   psubt->higher = (subentry *) 0;
   psubt->lower = (subentry *) 0;
@@ -88,7 +74,7 @@ char *fatal;
   psubt->lastmsg = msg;
   psubt->msginthread = 1;
   if (!(psubt->sub = alloc ((sublen) * sizeof(char))))
-    die_nomem(fatal);
+    die_nomem();
   cpto = psubt->sub;
   cpno = sublen;
   cpfrom = subject;
@@ -96,24 +82,22 @@ char *fatal;
   psubt->sublen = sublen;
 }
 
-static void newauth(pautht,author,authlen,msg,fatal)
+static void newauth(authentry *pautht,		/* entry for current message */
+		    const char *author,/* pointer to author string (not sz!) */
+		    unsigned int authlen,	/* lenth of author */
+		    unsigned long msg)		/* sz */
 /* Allocates space for author of length authlen+1 adding a terminal '\0' */
 /* and puts the pointer in pautht->auth. Analog to newsub().             */
-authentry *pautht;	/* entry for current message */
-char *author;		/* pointer to author string (not sz!) */
-unsigned int authlen;	/* lenth of author */
-unsigned long msg;
-char *fatal;		/* sz */
-
 {
-  register char *cpfrom, *cpto;
-  register unsigned int cpno;
+  const char *cpfrom;
+  char *cpto;
+  unsigned int cpno;
 
   pautht->higher = (subentry *) 0;
   pautht->lower = (subentry *) 0;
   pautht->firstmsg = msg;
   if (!(pautht->auth = alloc ((authlen) * sizeof(char))))
-    die_nomem(fatal);
+    die_nomem();
   cpto = pautht->auth;
   cpno = authlen;
   cpfrom = author;
@@ -121,20 +105,26 @@ char *fatal;		/* sz */
   pautht->authlen = authlen;
 }
 
-static void init_dummy(fatal)
-char *fatal;
+static void init_dummy(void)
 {
   unsigned int i;
 
-  if (!stralloc_ready(&dummyind,HASHLEN + 1)) die_nomem(fatal);
+  if (!stralloc_ready(&dummyind,HASHLEN + 1)) die_nomem();
   for (i = 0; i< HASHLEN; i++)
     dummyind.s[i] = 'a';
   dummyind.len = HASHLEN;
-  if (!stralloc_append(&dummyind," ")) die_nomem(fatal);
+  if (!stralloc_append(&dummyind," ")) die_nomem();
 }
 
-void idx_mkthreads(pmsgtable,psubtable,pauthtable,pdatetable,
-	msg_from,msg_to,msg_latest,locked,fatal)
+void idx_mkthreads(msgentry **pmsgtable,	/* table of message<->subject */
+		   subentry **psubtable,	/* subject no, len, str char * */
+		   authentry **pauthtable,	/* author no, len, str char* */
+		   dateentry **pdatetable,	/* message per date */
+		   unsigned long msg_from,	/* first message in range */
+		   unsigned long msg_to,	/* last message in range */
+		   unsigned long msg_latest,	/* latest message in archive (for locking) */
+		   int locked)			/* if already locked */
+
 /* Threads messages msg_from -> msg_to into pmsgtable & psubtable. When  */
 /* reading the latest index file (containing msg_latest) it locks the    */
 /* directory, unless it is already locked (as in digest creation).       */
@@ -145,17 +135,6 @@ void idx_mkthreads(pmsgtable,psubtable,pauthtable,pdatetable,
 /* missing entries, not necessarily reflecting missing archive files.    */
 /* This all to make ezmlm-get more robust to get maximal info out of     */
 /* corrupted archives.                                                   */
-
-  msgentry **pmsgtable;		/* table of message<->subject */
-  subentry **psubtable;		/* subject no, len, str char * */
-  authentry **pauthtable;	/* author no, len, str char * */
-  dateentry **pdatetable;	/* message per date */
-  unsigned long msg_from;	/* first message in range */
-  unsigned long msg_to;		/* last message in range */
-  unsigned long msg_latest;	/* latest message in archive (for locking) */
-  int locked;			/* if already locked */
-  char *fatal;			/* Program-specific */
-
 {
   unsigned long idxlatest;	/* need to lock for this (last) index file */
   unsigned long msg;		/* current msg number */
@@ -166,9 +145,10 @@ void idx_mkthreads(pmsgtable,psubtable,pauthtable,pdatetable,
   unsigned long ulmrange;	/* total # of messages in range */
   char *subject;		/* subject on line */
   unsigned int sublen;		/* length of subject */
-  char *auth;
-  unsigned int authlen;
-  unsigned int pos,posa;
+  char *auth = 0;
+  unsigned int authlen = 0;
+  unsigned int pos = 0;
+  unsigned int posa;
   unsigned long submax;		/* max subject num in subtable */
   subentry *psubnext;		/* points to next entry in subtable */
   subentry *psubt;		/* points to entry in subtable */
@@ -176,7 +156,7 @@ void idx_mkthreads(pmsgtable,psubtable,pauthtable,pdatetable,
   authentry *pautht;		/* points to entry in authtable */
   int fd;			/* index file handle */
   int flagmissingindex;		/* current index file is missing */
-  int flagauth;			/* read index entry has author info */
+  int flagauth = 0;		/* read index entry has author info */
   int hasauth;			/* current msg's entry has author info */
   msgentry *pmsgt;
   int res;
@@ -185,16 +165,16 @@ void idx_mkthreads(pmsgtable,psubtable,pauthtable,pdatetable,
   unsigned int datetablesize,datetableunit;
   unsigned int lastdate = 0;
   unsigned int thisdate;
-  register msgentry *x, *y;
+  msgentry *x, *y;
 
 				/* a few unnecessary sanity checks */
   if (msg_to > msg_latest)
     msg_to = msg_latest;
   if (msg_to < msg_from)
-    strerr_die2x(100,fatal,"Program error: bad range in idx_mkthreads");
+    strerr_die2x(100,FATAL,"Program error: bad range in idx_mkthreads");
   ulmrange = msg_to - msg_from + 1;
   if (!(*pmsgtable = (msgentry *) alloc(ulmrange * sizeof(msgentry))))
-        die_nomem(fatal);
+        die_nomem();
   y = *pmsgtable;
   x = y + ulmrange;		/* clear */
   while (--x >= y) {
@@ -204,14 +184,14 @@ void idx_mkthreads(pmsgtable,psubtable,pauthtable,pdatetable,
   }
 				/* max entries - acceptable waste for now */
   if (!(*psubtable = (subentry *) alloc((ulmrange+1) * sizeof(subentry))))
-        die_nomem(fatal);
+        die_nomem();
 
   if (!(*pauthtable = (authentry *) alloc((ulmrange+1) * sizeof(authentry))))
-        die_nomem(fatal);
+        die_nomem();
   datetableunit = DATENO * sizeof(dateentry);
   datetablesize = datetableunit;
   if (!(*pdatetable = (dateentry *) alloc(datetablesize)))
-        die_nomem(fatal);
+        die_nomem();
   datepos = 0;
   datemax = DATENO - 2;		/* entry 0 and end marker */
   lastdate = 0;
@@ -221,8 +201,8 @@ void idx_mkthreads(pmsgtable,psubtable,pauthtable,pdatetable,
   submax = 0;
   psubnext = *psubtable;	/* dummy node to get tree going. Basically, */
   psubt = &sdummy;		/* assure that subject > psubt-sub and that */
-  init_dummy(fatal);		/* below ok unless HASHLEN > 40 */
-  psubt->sub = "                                       ";
+  init_dummy();			/* below ok unless HASHLEN > 40 */
+  psubt->sub = (char*)"                                       ";
   psubt->sublen = 40;		/* there is something to hold psubt->higher */
   psubt->higher = (subentry *) 0;
   psubt->lower = (subentry *) 0;
@@ -234,12 +214,12 @@ void idx_mkthreads(pmsgtable,psubtable,pauthtable,pdatetable,
   pautht->lower = (authentry *) 0;
   for (idx = msg_from / 100; idx <= idxto; idx++) {
 				/* make index file name */
-    if (!stralloc_copys(&line,"archive/")) die_nomem(fatal);
-    if (!stralloc_catb(&line,strnum,fmt_ulong(strnum,idx))) die_nomem(fatal);
-    if (!stralloc_cats(&line,"/index")) die_nomem(fatal);
-    if (!stralloc_0(&line)) die_nomem(fatal);
+    if (!stralloc_copys(&line,"archive/")) die_nomem();
+    if (!stralloc_catb(&line,strnum,fmt_ulong(strnum,idx))) die_nomem();
+    if (!stralloc_cats(&line,"/index")) die_nomem();
+    if (!stralloc_0(&line)) die_nomem();
     if (!locked && idx == idxlatest)
-      lockup(fatal);
+      lockup();
     flagmissingindex = 0;
     fd = open_read(line.s);
     if (fd == -1) {
@@ -247,7 +227,7 @@ void idx_mkthreads(pmsgtable,psubtable,pauthtable,pdatetable,
 					/* but the lists is supposedly indexed*/
         flagmissingindex = 1;
       } else
-        strerr_die4sys(111,fatal,ERR_OPEN,line.s,": ");
+        strerr_die2sys(111,FATAL,MSG1(ERR_OPEN,line.s));
     } else
       substdio_fdbuf(&ssindex,read,fd,indexbuf,sizeof(indexbuf));
 
@@ -261,14 +241,14 @@ void idx_mkthreads(pmsgtable,psubtable,pauthtable,pdatetable,
       if (!flagmissingindex && (msg > tmpmsg)) {
         flagauth = 0;
         if (getln(&ssindex,&line,&match,'\n') == -1)
-          strerr_die3sys(111,fatal,ERR_READ,"index: ");
+          strerr_die2sys(111,FATAL,MSG1(ERR_READ,"index"));
         if (!match)
           flagmissingindex = 1;
         else {
           pos = scan_ulong(line.s,&tmpmsg);
           if (line.s[pos++] == ':') {
             if (getln(&ssindex,&authline,&match,'\n') == -1)
-              strerr_die3sys(111,fatal,ERR_READ,"index: ");
+              strerr_die2sys(111,FATAL,MSG1(ERR_READ,"index"));
             if (!match)
               flagmissingindex = 1;
             else {
@@ -284,7 +264,7 @@ void idx_mkthreads(pmsgtable,psubtable,pauthtable,pdatetable,
         subject = line.s + pos;
         sublen = line.len - pos;
 	if (sublen <= HASHLEN)
-	  strerr_die2x(100,fatal,ERR_BAD_INDEX);
+	  strerr_die2x(100,FATAL,MSG(ERR_BAD_INDEX));
         hasauth = flagauth;
       } else {
         subject = dummyind.s;
@@ -297,7 +277,7 @@ void idx_mkthreads(pmsgtable,psubtable,pauthtable,pdatetable,
           if (psubt->higher)
             psubt = psubt->higher;
           else {
-            newsub(psubnext,subject,sublen,msg,fatal);
+            newsub(psubnext,subject,sublen,msg);
             psubt->higher = psubnext;
             psubt = psubnext;
             psubnext++;
@@ -307,7 +287,7 @@ void idx_mkthreads(pmsgtable,psubtable,pauthtable,pdatetable,
           if (psubt->lower)
             psubt = psubt->lower;
           else {
-            newsub(psubnext,subject,sublen,msg,fatal);
+            newsub(psubnext,subject,sublen,msg);
             psubt->lower = psubnext;
             psubt = psubnext;
             psubnext++;
@@ -333,9 +313,9 @@ void idx_mkthreads(pmsgtable,psubtable,pauthtable,pdatetable,
 	    lastdate = pmsgt->date;
 	    if (datepos >= datemax) {		/* more space */
 	      datemax += DATENO;
-	      if (!(*pdatetable = (dateentry *) alloc_re(*pdatetable,
-			datetablesize,datetablesize+datetableunit)))
-		die_nomem(fatal);
+	      if (!alloc_re((void**)pdatetable,
+			    datetablesize,datetablesize+datetableunit))
+		die_nomem();
 	    }
 	    (*pdatetable)[datepos].msg = msg;	/* first msg this mo */
 	    (*pdatetable)[datepos].date = lastdate;
@@ -358,7 +338,7 @@ void idx_mkthreads(pmsgtable,psubtable,pauthtable,pdatetable,
             if (pautht->higher)
               pautht = pautht->higher;
             else {
-              newauth(pauthnext,auth,authlen,msg,fatal);
+              newauth(pauthnext,auth,authlen,msg);
               pautht->higher = pauthnext;
               pautht = pauthnext;
               pauthnext++;
@@ -368,7 +348,7 @@ void idx_mkthreads(pmsgtable,psubtable,pauthtable,pdatetable,
             if (pautht->lower)
               pautht = pautht->lower;
             else {
-              newauth(pauthnext,auth,authlen,msg,fatal);
+              newauth(pauthnext,auth,authlen,msg);
               pautht->lower = pauthnext;
               pautht = pauthnext;
               pauthnext++;
@@ -398,22 +378,18 @@ void idx_mkthreads(pmsgtable,psubtable,pauthtable,pdatetable,
 }
 
 
-void idx_mkthread(pmsgtable,psubtable,pauthtable,msg_from,msg_to,msg_master,
-		msg_latest,locked,fatal)
+void idx_mkthread(msgentry **pmsgtable,		/* pointer to table of message<->subject */
+		  subentry **psubtable,		/* ptr to tbl of subject no, len, str char * */
+		  authentry **pauthtable,
+		  unsigned long msg_from,	/* first message in range */
+		  unsigned long msg_to,		/* last message in range */
+		  unsigned long msg_master,	/* master message for single thread, else 0*/
+		  unsigned long msg_latest,	/* latest message in archive (for locking) */
+		  int locked)			/* if already locked */
+
 /* Works like idx_mkthreads, except that it finds the subject for message   */
 /* msg_master, then identifies messages in the range that have the same     */
 /* subject. msgtable entries with subject 0 do not match, with '1' do match.*/
-
-msgentry **pmsgtable;		/* pointer to table of message<->subject */
-subentry **psubtable;		/* ptr to tbl of subject no, len, str char * */
-authentry **pauthtable;
-unsigned long msg_from;		/* first message in range */
-unsigned long msg_to;		/* last message in range */
-unsigned long msg_latest;	/* latest message in archive (for locking) */
-unsigned long msg_master;	/* master message for single thread, else 0*/
-int locked;			/* if already locked */
-char *fatal;			/* Program-specific */
-
 {
   unsigned long idxlatest;	/* need to lock for this (last) index file */
   unsigned long idxto;		/* index for last msg in range */
@@ -433,12 +409,12 @@ char *fatal;			/* Program-specific */
   int fd;			/* index file handle */
   int match;
   msgentry *pmsgt;
-  register msgentry *x,*y;
+  msgentry *x,*y;
 
   if ((ulmrange = msg_to - msg_from +1) <= 0)
-    strerr_die2x(100,fatal,"Program error: bad range in idx_mkthreads");
+    strerr_die2x(100,FATAL,"Program error: bad range in idx_mkthreads");
   if (!(*pmsgtable = (msgentry *) alloc(ulmrange * sizeof(msgentry))))
-         die_nomem(fatal);
+         die_nomem();
   y = *pmsgtable;
   x = y + ulmrange;
   while (--x >= y) {
@@ -448,15 +424,15 @@ char *fatal;			/* Program-specific */
   }
 
   if (!(*psubtable = (subentry *) alloc(2 * sizeof(subentry))))
-          die_nomem(fatal);
+          die_nomem();
 
   if (!(*pauthtable = (authentry *) alloc((ulmrange + 1) * sizeof(authentry))))
-          die_nomem(fatal);
+          die_nomem();
 
   pauthnext = *pauthtable;
   pautht = &adummy;
   init_dummy();
-  pautht->auth = "                     ";
+  pautht->auth = (char*)"                     ";
   pautht->authlen = 21;
   pautht->higher = (authentry *) 0;
   pautht->lower = (authentry *) 0;
@@ -465,25 +441,25 @@ char *fatal;			/* Program-specific */
   idx = msg_master / 100;	/* index for master subject */
 
 				/* Get master subject */
-  if (!stralloc_copys(&line,"archive/")) die_nomem(fatal);
-  if (!stralloc_catb(&line,strnum,fmt_ulong(strnum,idx))) die_nomem(fatal);
-  if (!stralloc_cats(&line,"/index")) die_nomem(fatal);
-  if (!stralloc_0(&line)) die_nomem(fatal);
+  if (!stralloc_copys(&line,"archive/")) die_nomem();
+  if (!stralloc_catb(&line,strnum,fmt_ulong(strnum,idx))) die_nomem();
+  if (!stralloc_cats(&line,"/index")) die_nomem();
+  if (!stralloc_0(&line)) die_nomem();
   ffound = 0;
   if (!locked && idx == idxlatest)
-    lockup(fatal);
+    lockup();
   fd = open_read(line.s);
   psubt = *psubtable;
   if (fd == -1) {
     if (errno != error_noent)
-      strerr_die4sys(111,fatal,ERR_OPEN,line.s,": ");
+      strerr_die2sys(111,FATAL,MSG1(ERR_OPEN,line.s));
     else
-      strerr_die2x(111,fatal,ERR_NOINDEX);	/* temp - admin can fix! */
+      strerr_die2x(111,FATAL,MSG(ERR_NOINDEX));	/* temp - admin can fix! */
   } else {
     substdio_fdbuf(&ssindex,read,fd,indexbuf,sizeof(indexbuf));
     for(;;) {
       if (getln(&ssindex,&line,&match,'\n') == -1)
-          strerr_die3sys(111,fatal,ERR_OPEN,"index: ");
+          strerr_die2sys(111,FATAL,MSG1(ERR_OPEN,"index"));
       if (!match)
         break;
       pos=scan_ulong(line.s,&msg);
@@ -493,14 +469,14 @@ char *fatal;			/* Program-specific */
       } else
         flagauth = 0;
       if (msg == msg_master) {
-        newsub(psubt,line.s+pos,line.len-pos,msg,fatal);
+        newsub(psubt,line.s+pos,line.len-pos,msg);
 					/* need to update msg later! */
         ffound = 1;
         break;
       }
       if (flagauth) {			/* skip author line */
         if (getln(&ssindex,&line,&match,'\n') == -1)
-          strerr_die3sys(111,fatal,ERR_OPEN,"index: ");
+          strerr_die2sys(111,FATAL,MSG1(ERR_OPEN,"index"));
       if (!match)
         break;
       }
@@ -510,24 +486,24 @@ char *fatal;			/* Program-specific */
   if (!locked && idx == idxlatest)
     unlock();
   if (!ffound)
-      strerr_die2x(100,fatal,ERR_NOINDEX);
+      strerr_die2x(100,FATAL,MSG(ERR_NOINDEX));
   for (idx = msg_from / 100; idx <= idxto; idx++) {
 		/* make index file name */
-    if (!stralloc_copys(&line,"archive/")) die_nomem(fatal);
-    if (!stralloc_catb(&line,strnum,fmt_ulong(strnum,idx))) die_nomem(fatal);
-    if (!stralloc_cats(&line,"/index")) die_nomem(fatal);
-    if (!stralloc_0(&line)) die_nomem(fatal);
+    if (!stralloc_copys(&line,"archive/")) die_nomem();
+    if (!stralloc_catb(&line,strnum,fmt_ulong(strnum,idx))) die_nomem();
+    if (!stralloc_cats(&line,"/index")) die_nomem();
+    if (!stralloc_0(&line)) die_nomem();
     if (!locked && idx == idxlatest)
-      lockup(fatal);
+      lockup();
     fd = open_read(line.s);
     if (fd == -1) {
       if (errno != error_noent)
-        strerr_die4sys(111,fatal,ERR_OPEN,line.s,": ");
+        strerr_die2sys(111,FATAL,MSG1(ERR_OPEN,line.s));
     } else {
       substdio_fdbuf(&ssindex,read,fd,indexbuf,sizeof(indexbuf));
       for(;;) {
         if (getln(&ssindex,&line,&match,'\n') == -1)
-          strerr_die3sys(111,fatal,ERR_READ,"index: ");
+          strerr_die2sys(111,FATAL,MSG1(ERR_READ,"index"));
         if (!match)
           break;
         pos=scan_ulong(line.s,&msg);
@@ -535,7 +511,7 @@ char *fatal;			/* Program-specific */
           pos++;
           flagauth = 1;
           if (getln(&ssindex,&authline,&match,'\n') == -1)
-            strerr_die3sys(111,fatal,ERR_READ,"index: ");
+            strerr_die2sys(111,FATAL,MSG1(ERR_READ,"index"));
           if (!match)
             break;
         } else
@@ -570,7 +546,7 @@ char *fatal;			/* Program-specific */
 	        if (pautht->higher)
 	          pautht = pautht->higher;
 	        else {
-	          newauth(pauthnext,auth,authlen,msg,fatal);
+	          newauth(pauthnext,auth,authlen,msg);
 	          pautht->higher = pauthnext;
 	          pautht = pauthnext;
 	          pauthnext++;
@@ -580,7 +556,7 @@ char *fatal;			/* Program-specific */
 	        if (pautht->lower)
 	          pautht = pautht->lower;
 	        else {
-	          newauth(pauthnext,auth,authlen,msg,fatal);
+	          newauth(pauthnext,auth,authlen,msg);
 	          pautht->lower = pauthnext;
 	          pautht = pauthnext;
 	          pauthnext++;
@@ -606,28 +582,26 @@ char *fatal;			/* Program-specific */
   pauthnext->auth = (char *) 0;	/* end of table marker */
 }
 
-void idx_mklist(pmsgtable,psubtable,pauthtable,msg_from,msg_to,fatal)
+void idx_mklist(msgentry **pmsgtable,	/* pointer to table of message<->subject */
+		subentry **psubtable,	/* ptr to tbl of subject no, len, str char * */
+		authentry **pauthtable,
+		unsigned long msg_from,		/* first message in range */
+		unsigned long msg_to)		/* last message in range */
 /* Like mkthreads, except that it works without a subject index. The result */
 /* is just a dummy subject and a sequential list of messages. This to allow */
 /* use of the same routines when creating digest from lists that have no    */
 /* subject index (for whatever reason). */
-msgentry **pmsgtable;		/* pointer to table of message<->subject */
-subentry **psubtable;		/* ptr to tbl of subject no, len, str char * */
-authentry **pauthtable;
-unsigned long msg_from;		/* first message in range */
-unsigned long msg_to;		/* last message in range */
-char *fatal;			/* Program-specific */
 {
   unsigned long ulmrange;
-  register msgentry *x,*y;
+  msgentry *x,*y;
   subentry *psubt;
   authentry *pautht;
 
   if ((ulmrange = msg_to - msg_from +1) <= 0)
-    strerr_die2x(111,fatal,"bad range in idx_mkthreads :");
+    strerr_die2x(111,FATAL,"bad range in idx_mkthreads :");
 
   if (!(*pmsgtable = (msgentry *) alloc(ulmrange * sizeof(msgentry))))
-         die_nomem(fatal);
+         die_nomem();
 
   y = *pmsgtable;
   x = y + ulmrange;
@@ -638,25 +612,26 @@ char *fatal;			/* Program-specific */
   }
 
   if (!(*psubtable = (subentry *) alloc(2 * sizeof(subentry))))
-          die_nomem(fatal);
+          die_nomem();
   psubt = *psubtable;
-  newsub(psubt,dummyind.s,dummyind.len,msg_from,fatal);
+  newsub(psubt,dummyind.s,dummyind.len,msg_from);
   psubt->lastmsg = msg_to;
   ++psubt;
   psubt->sub = (char *) 0;
   if (!(*pauthtable = (authentry *) alloc(sizeof(authentry))))
-          die_nomem(fatal);	/* nodata. Avoid dangling ptr. */
+          die_nomem();		/* nodata. Avoid dangling ptr. */
   pautht = *pauthtable;
   pautht->auth = 0;		/* tells app that there are no author data */
   pautht->higher = (authentry *) 0;
   pautht->lower = (authentry *) 0;
 }
 
-void idx_destroythread(msgtable,subtable,authtable)
+void idx_destroythread(msgentry *msgtable,
+		       subentry *subtable,
+		       authentry *authtable)
 /* Frees space allocated by idxthread routines. This is needed only if */
 /* one does several threadings in one program run. Otherwise, exit()   */
 /* should free all allocated memory, which will be faster. */
-msgentry *msgtable; subentry *subtable; authentry *authtable;
 {
   subentry *psubt;
   authentry *pautht;
